#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <sys/stat.h>
#include <string.h>

/* amirunning.c
 * Simple little program to be called from shell scripts to cycle through
 * proc directories looking for the same invocation
 *
 * 2009-04-08 ed AT s5h DOT net - version 0.1 beta code
 * Use at your own risk. Author not responsible for use.
 *
 * Updates at http://www.s5h.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1
#define EXIT_RUNNING 2

#define BUFFER 65535
#define ONEMEG 1048576



int size_of_file( char *filename ) {
	struct stat s_data;

	if( stat( filename, &s_data ) != 0 ) {
		return( 0 );
	}

	return( s_data.st_size );	
}

// so many places here exit without freeing d3, but who gives a shit
int already_running( int parent_pid ) {
	DIR *d;
	struct dirent *r_info;
	FILE *parent_process_fd;
	char path[BUFFER];
	char d1[ONEMEG],d2[ONEMEG],*d3;
	int s1,s2;
	int size_of_parent;
	
	// zero the memory that we'll copy later
	d3 = calloc( 1, ONEMEG );
	if( d3 == NULL ) {
		perror( "could not allocate a meg of memory" );
		return( EXIT_FAILURE );
	}

	d = opendir( "/proc" );
	if( errno != 0 ) {
		perror( "Could not open /proc" );
		free(d3);
		return( EXIT_FAILURE );
	}

	snprintf( path, BUFFER, "/proc/%d/cmdline", parent_pid );

	size_of_parent = size_of_file( path );

	parent_process_fd = fopen( path, "r" );
	if( parent_process_fd == NULL ) {
		free(d3);
		fprintf( stderr, "Could not open %s", path );
		return( EXIT_FAILURE );
	}

	memcpy( d1, d3, ONEMEG );
	fread( d1, ONEMEG, 1, parent_process_fd );
	// s1 = fgetpos( parent_process_fd, SEEK_SET );
	s1 = ftell( parent_process_fd );

	while( ( r_info = readdir( d ) ) != NULL ) {
		int a;
		char cmdline[BUFFER];
		FILE *cur_pid;

		if( r_info->d_type != DT_DIR ) {
			continue ;
		}

		a = atoi( r_info->d_name );
		if( a == 0 || a == parent_pid ) {
			continue;
		}
		
		// fprintf( stdout, "%.40s\t%.10s\n", r_info->d_name, ( r_info->d_type == DT_DIR ? "D" : "" ) );

		snprintf( cmdline, BUFFER, "/proc/%d/cmdline", a );
		cur_pid = fopen( cmdline, "r" );
		
		if( cur_pid == NULL ) {
			continue;
		}

		memcpy( d2, d3, ONEMEG );
		fread( d2, ONEMEG, 1, cur_pid );
		// s2 = fgetpos( cur_pid, SEEK_SET );
		s2 = ftell( cur_pid );
		
		fclose( cur_pid );

		//s1 = strlen( d1 );
		//s2 = strlen( d2 );

		// fprintf( stderr, "[%s] [%s] [%d]\n", d1, d2, ( size_of_parent > s2 ? size_of_parent : s2 ) );
		// fprintf( stderr, "[%s] [%s] [%d]\n", d1, d2, ( size_of_parent > s2 ? size_of_parent : s2 ) );

		// fwrite( d1, s1, 1, stdout );
		// fprintf( stderr, "\n" );


		if( bcmp( d1, d2, s1 > s2 ? s1 : s2 ) == 0 ) {
			// fprintf( stdout, " ++ matched [%s] [%s] [%d]\n", d1, d2, ( size_of_parent > s2 ? size_of_parent : s2 ) );
			// fwrite( d1, s1, 1, stdout );
			// fprintf( stdout, "\n" );
			free(d3);
			return( EXIT_RUNNING );
		}


		//if( memcmp( d1, d2, size_of_parent > s2 ? size_of_parent : s2 ) == 0 ) {
		//	fprintf( stderr, "matched [%s] [%s] [%d]\n", d1, d2, ( size_of_parent > s2 ? size_of_parent : s2 ) );
		//	return( EXIT_RUNNING );
		//}
	}

	closedir( d );
	fclose( parent_process_fd );
	free(d3);
	return( EXIT_SUCCESS );
}

int main() {
	pid_t parent_id;
	
	parent_id = getppid();

	if( parent_id == 0 ) {
		return( EXIT_SUCCESS );
	}

	return( already_running( (int)parent_id ) );
}

